@@ -59,6 +59,7 @@ struct _EphyHistoryDialog {
   GtkWidget *search_bar;
   GtkWidget *search_entry;
   GtkWidget *history_presentation_stack;
+  GtkWidget *history_scrolled_window;
   GtkWidget *listbox;
   GtkWidget *loading_spinner;
   GtkWidget *empty_history_message;
@@ -115,12 +116,12 @@ update_ui_state (EphyHistoryDialog *self)
   } else {
     if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->search_button))) {
       if (has_data && self->has_search_results)
-        gtk_stack_set_visible_child (history_presentation_stack, self->listbox);
+        gtk_stack_set_visible_child (history_presentation_stack, self->history_scrolled_window);
       else
         gtk_stack_set_visible_child (history_presentation_stack, self->no_search_results_message);
     } else {
       if (has_data)
-        gtk_stack_set_visible_child (history_presentation_stack, self->listbox);
+        gtk_stack_set_visible_child (history_presentation_stack, self->history_scrolled_window);
       else
         gtk_stack_set_visible_child (history_presentation_stack, self->empty_history_message);
     }
@@ -632,6 +633,16 @@ on_key_press_event (EphyHistoryDialog *self,
     }
   }
 
+  if (key->keyval == GDK_KEY_Escape &&
+      !hdy_search_bar_get_search_mode (search_bar)) {
+    if (self->selection_active)
+      set_selection_active (self, FALSE);
+    else
+      gtk_window_close (GTK_WINDOW (self));
+
+    return GDK_EVENT_STOP;
+  }
+
   /* Edge case: Shift + Enter in selection mode
    * Pressing simply Enter without any modifiers activates the focused row,
    * but pressing Enter with modifiers doesn't do anything.
@@ -904,6 +915,7 @@ ephy_history_dialog_class_init (EphyHistoryDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, EphyHistoryDialog, search_bar);
   gtk_widget_class_bind_template_child (widget_class, EphyHistoryDialog, search_entry);
   gtk_widget_class_bind_template_child (widget_class, EphyHistoryDialog, history_presentation_stack);
+  gtk_widget_class_bind_template_child (widget_class, EphyHistoryDialog, history_scrolled_window);
   gtk_widget_class_bind_template_child (widget_class, EphyHistoryDialog, listbox);
   gtk_widget_class_bind_template_child (widget_class, EphyHistoryDialog, loading_spinner);
   gtk_widget_class_bind_template_child (widget_class, EphyHistoryDialog, empty_history_message);
@@ -954,6 +966,7 @@ ephy_history_dialog_init (EphyHistoryDialog *self)
   self->cancellable = g_cancellable_new ();
   self->urls = NULL;
   self->sorter_source = 0;
+  self->is_selection_empty = TRUE;
 
   hdy_search_bar_connect_entry (HDY_SEARCH_BAR (self->search_bar), GTK_ENTRY (self->search_entry));
 
@@ -969,4 +982,7 @@ ephy_history_dialog_init (EphyHistoryDialog *self)
 
   gtk_widget_set_tooltip_text (self->clear_all_button, tooltip);
   set_is_loading (self, TRUE);
+
+  hdy_status_page_set_icon_name (HDY_STATUS_PAGE (self->empty_history_message),
+                                 APPLICATION_ID "-symbolic");
 }
